// gvgbs.cpp: implementation of the gvgbs class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "gvgbs.h"
#include <math.h>
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//===========GENERAL GRAPHIC FUNTIONS==================

void gSetCursor(int n)
{
	HCURSOR hcursor;
	if(n==1) hcursor=AfxGetApp()->LoadStandardCursor(IDC_WAIT);
	else hcursor=AfxGetApp()->LoadStandardCursor(IDC_ARROW);
	SetCursor(hcursor);
}

void gSetFont(int Height, int Fat, BOOL Italic, BOOL Orientation, CFont *F)
{
	Fat*=100; if(Fat>1000) Fat=Fat%1000;
	if(Italic) Italic=1; else Italic=0;
	int angle;
	if(Orientation) angle=900; else angle=0;
	F->DeleteObject(); //delete previous version
	F->CreateFont (Height,0,angle,angle,Fat,Italic,0,0,ANSI_CHARSET,
		OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,PROOF_QUALITY,
		VARIABLE_PITCH|FF_ROMAN,"Times New Roman");
}






//////////////////////////////////////////////////////////////////////
// MAIN CLASS FUNTIONS : Construction/Destruction
//////////////////////////////////////////////////////////////////////

gvgbs::gvgbs()
{
	Gtitle="gvgbs\0";
	Gvisible=TRUE;
	rxleft=50;
	rxright=90;
	rytop=10;
	rybottom=90;
	xmin=-0.5;
	xmax=1.0;
	ymin=-1.0;
	ymax=1.0;	
	refPixel=(100,100);
	
	ShowTicks=TRUE;
	ShowLabels=TRUE;
	DoubleBorder=TRUE;
	nxtick=4;
	nytick=8;
	wytick=4;
	wxtick=4;
	dxtick=2;
	dytick=2;
	RoundScales=TRUE;	
}


gvgbs::~gvgbs()
{

}

void gvgbs::SetScreen(int xl, int yt, int xr, int yb)
{
	if(xl<0||xl>100||xl>=xr) xl=1;
	if(xr<0||xr>100||xr<=xl) xr=100;
	if(yt<0||yt>100||yt>=yb) yt=1;
	if(yb<0||yb>100||yb<=yt) yb=100;
	rxleft=xl; rxright=xr; rytop=yt; rybottom=yb;
}


void gvgbs::SetWorld(double xm, double ym, double xM, double yM)
{
	if(RoundScales) {GetRinterval(&xm,&xM);GetRinterval(&ym,&yM);}
	xmin=xm;ymin=ym;xmax=xM;ymax=yM;

}

void gvgbs::Fixing(RECT RR)
{
	RECT R;
	double rx,ry;
	rx=(double)(RR.right-RR.left)/100;
	ry=(double)(RR.bottom-RR.top)/100;
	R.left=(int)(rxleft*rx+0.5);
	R.right=(int)(rxright*rx+0.5);
	R.top=(int)(rytop*ry+0.5);
	R.bottom=(int)(rybottom*ry+0.5);
	
	fx=(R.right-R.left)/(xmax-xmin);
	cx=fx*xmin-R.left;
	fy=(R.top-R.bottom)/(ymax-ymin);
	cy=fy*ymin-R.bottom;
}


CPoint gvgbs::gpixel(double x, double y, BOOL areaprotection)
{
	CPoint pp;
	if(!Gvisible) {pp.x=pp.y=0;return pp;}
	if(areaprotection&&(x>xmax||x<xmin||y>ymax||y<ymin)) {pp.x=pp.y=0;}
	else
	{	
		pp.x=(int)(fx*x-cx+0.5);
		pp.y=(int)(fy*y-cy+0.5);
	}
	return pp;
}


RECT gvgbs::gbrect()
{
	CPoint p; RECT R;
	if(!Gvisible) {R.left=R.bottom=R.right=R.top=0; return R;}
	p=gpixel(xmin,ymax,0); R.left=p.x; R.top=p.y;
	p=gpixel(xmax,ymin,0); R.right=p.x; R.bottom=p.y;
	return R;
}

void gvgbs::GetScreen(int *x1, int *y1, int *x2, int *y2)
{
	*x1=rxleft;
	*y1=rytop;
	*x2=rxright;
	*y2=rybottom;
}

void gvgbs::GetWorld(double *xm, double *ym, double *xM, double *yM)
{
	*xm=xmin; *xM=xmax; *ym=ymin; *yM=ymax;
}

void gvgbs::GetLabel(BOOL wide, BOOL top,int *x, int *y, CString *ss)
{
	CString ll;
	if(!Gvisible) {*x=0; *y=0; *ss=""; return;}
	if(wide)
	ll.Format("> X(%0.7f,%0.7f)Y(%0.7f,%0.7f)",xmin,ymin,xmax,ymax);
	else
	ll.Format("> X(%0.3f,%0.3f)Y(%0.3f,%0.3f)",xmin,ymin,xmax,ymax);
	*ss=Gtitle+ll;
	CPoint p;
	if(top)
	{
		p=gpixel(xmin,ymax,0); *x=p.x; *y=p.y-18;
	}
	else
	{
		p=gpixel(xmin,ymin,0); *x=p.x; *y=p.y+2;
	}
}



BOOL gvgbs::gpoint(CPoint P, double *x, double *y)
//get data point (x,y) from screen point P
{
	*x=(P.x+cx)/fx;
	*y=(P.y+cy)/fy;
	if(*x<xmin||*x>xmax||*y<ymin||*y>ymax) return 0;
	else return 1;
}

BOOL gvgbs::putpixel(HDC dc, double x, double y, COLORREF col, int ap)
{
	CPoint pp;
	if(!Gvisible) return 0;
	if(ap&&(x>xmax||x<xmin||y>ymax||y<ymin)) return 0;
	pp.x=(int)(fx*x-cx+0.5);
	pp.y=(int)(fy*y-cy+0.5);
	SetPixel(dc,pp.x,pp.y,col);	
	return 1;
}


BOOL gvgbs::putBpixel(HDC dc, double x, double y, int rad, COLORREF col, BOOL ap)
{
	CPoint pp;
	if(!Gvisible) return 0;
	if(ap&&(x>xmax||x<xmin||y>ymax||y<ymin)) return 0;
	pp.x=(int)(fx*x-cx+0.5);
	pp.y=(int)(fy*y-cy+0.5);
	for(int k=-rad;k<=rad;k++)
		for(int l=-rad;l<=rad;l++)
			SetPixel(dc,pp.x+k,pp.y+l,col);	
	return 1;
}


void gvgbs::SetRefPixel(double x, double y)
{
	refPixel=gpixel(x,y,0);
}

void gvgbs::putline(HDC dc, double x, double y, BOOL prot)
{
	CPoint pp;
	if(!Gvisible) return;
	if(prot&&(x>xmax||x<xmin||y>ymax||y<ymin)) return;
	pp.x=(int)(fx*x-cx+0.5);
	pp.y=(int)(fy*y-cy+0.5);
	MoveToEx(dc,refPixel.x,refPixel.y,0);
	LineTo(dc,pp.x,pp.y);
	refPixel=pp;
}



void gvgbs::DrawTicks(HDC dc)
{

	CPoint p,p1,p2; RECT R;
	if(!Gvisible) return;
	//Border;
	p=gpixel(xmin,ymax,0); R.left=p.x; R.top=p.y;
	p=gpixel(xmax,ymin,0); R.right=p.x; R.bottom=p.y;
	if(DoubleBorder) Rectangle(dc,R.left-1,R.top-1,R.right+1,R.bottom+1);
	Rectangle(dc,R.left,R.top,R.right,R.bottom);
	if(!ShowTicks) return;
	//X-Axis Ticks
	double Dw=(ymax-ymin)*wxtick*0.01;
	double Dr=(xmax-xmin)/nxtick;
	double rlen;
	for(int i=0;i<=nxtick;i++)
	{
		if(i%2==0) rlen=1.0; else rlen=0.7;
		double r=xmin+i*Dr; 
		double v1=ymin; double v2=ymin-Dw*rlen;
		p1=gpixel(r,v1,0); p2=gpixel(r,v2,0);
		MoveToEx(dc,p1.x,p1.y,0); LineTo(dc,p2.x,p2.y);
	}
	//Y-Axis Ticks
	Dr=(ymax-ymin)/nytick;
	Dw=(xmax-xmin)*wytick*0.01;
	for(i=0;i<=nytick;i++)
	{
		if(i%2==0) rlen=1.0; else rlen=0.7;
		double r=ymin+i*Dr; 
		double v1=xmin; double v2=xmin-Dw*rlen;
		p1=gpixel(v1,r,0); p2=gpixel(v2,r,0);
		MoveToEx(dc,p1.x,p1.y,0); LineTo(dc,p2.x,p2.y);
	}
	if(!ShowLabels) return;
	//X-Labels
	Dw=(ymax-ymin)*wxtick*0.01;
	Dr=(xmax-xmin)/nxtick;
	for(i=0;i<=nxtick;i+=2)
	{
		double r=xmin+i*Dr; if(r>xmax+1.0e-8) break;
		double v=ymin-Dw;
		p1=gpixel(r,v,0); p1.y-=3;
		CString ss;
		if(dxtick==0) ss.Format("%.0f",r);
		if(dxtick==1) ss.Format("%.1f",r);
		if(dxtick==2) ss.Format("%.2f",r);
		if(dxtick==3) ss.Format("%.3f",r);
		if(dxtick==4) ss.Format("%.4f",r);
		int l=ss.GetLength();
		TextOut(dc,p1.x-l*4,p1.y,ss,l);
	}
	//Y-Labels
	Dw=(xmax-xmin)*wytick*0.01;
	Dr=(ymax-ymin)/nytick;
	for(i=0;i<=nytick;i+=2)
	{
		double r=ymin+i*Dr; if(r>ymax+1.0e-8) break;
		double v=xmin-Dw;
		p1=gpixel(v,r,0); 
		CString ss;
		if(dytick==0) ss.Format("%.0f",r);
		if(dytick==1) ss.Format("%.1f",r);
		if(dytick==2) ss.Format("%.2f",r);
		if(dytick==3) ss.Format("%.3f",r);
		if(dytick==4) ss.Format("%.4f",r);
		int l=ss.GetLength();
		TextOut(dc,p1.x-l*7,p1.y-8,ss,l);
	}
}

void gvgbs::puttitle(HDC dc)
{
	int len=Gtitle.GetLength();
	CPoint p=gpixel(xmin+(xmax-xmin)/2,ymax,0); 	
	p.x-=len*4; p.y-=25;
	TextOut(dc,p.x,p.y,Gtitle,len);
}

void gvgbs::puttext(HDC dc, double x, double y, CString ss)
{
	CPoint p=gpixel(x,y,0);
	TextOut(dc,p.x,p.y,ss,ss.GetLength());
}



void gvgbs::SetAxis(HDC dc, double xc, double yc)
{
	CPoint p=refPixel;

	SetRefPixel(xmin,yc); putline(dc,xmax,yc,0);
	SetRefPixel(xc,ymin); putline(dc,xc,ymax,0);
	refPixel=p;
}


void gvgbs::SetXYticks(int nx, int ny, int wx, int wy, int dx, int dy)
{
	nxtick=nx; nytick=ny;
	wxtick=wx; wytick=wy;
	dxtick=dx; dytick=dy;
}

void gvgbs::GetXYticks(int *nx, int *ny, int *wx, int *wy, int *dx, int *dy)
{
	*nx=nxtick; *ny=nytick;
	*wx=wxtick; *wy=wytick;
	*dx=dxtick; *dy=dytick;
}

void gvgbs::Fulfil(HDC dc, RECT R)
{
	Fixing(R);
	DrawTicks(dc);
	puttitle(dc);
}

void gvgbs::GetRinterval(double *x1a, double *x2a)
{
	double acc=0.0555;
	double x1=*x1a; double x2=*x2a;
	double D=x2-x1; if(D<=0.000000001) return;
	int n=(int)(log10(D)+0.5) -1;
	double p=pow(10,n);
	*x1a=floor(x1/p+acc);		*x1a*=p;
	*x2a=ceil(x2/p-acc);		*x2a*=p;
}
